@@ -43,8 +43,8 @@ package Lock::Server;
     use Lock::Server::Client;
 
     my $lockServer = new Lock::Server( {
-       lock_timeout         => 10, #seconds. default is 3
-       lock_attempt_timeout => 12, #seconds. default is 4
+       lock_timeout         => 10, #microsecondsseconds. default is 3000
+       lock_attempt_timeout => 12, #microseconds. default is 4000
        port                 => 888, #default is 8004
        host                 => 'localhost', #default 127.0.0.1
     } );
@@ -92,16 +92,19 @@ no warnings 'uninitialized';
 
 use Data::Dumper;
 
-use Socket;
+use IO::Select;
+use IO::Socket;
+
 use IO::Socket::INET;
 use POSIX ":sys_wait_h";
+use Time::HiRes qw(ualarm usleep);
 
 use vars qw($VERSION);
 
-$VERSION = '1.72';
+$VERSION = '1.73';
 
 
-$Lock::Server::DEBUG = 1;
+$Lock::Server::DEBUG = 0;
 
 =head2 Lock::Server::new( $args )
 
@@ -127,12 +130,13 @@ sub new {
         host                 => $args->{host} || '127.0.0.1',
         port                 => $args->{port} || 8004,
         allow_shutdown       => $args->{allow_shutdown},
+        max_connections      => $args->{max_connections} || 10,
         _pids                => {},
         _id2pid              => {},
         _locks               => {},
         _locker_counts       => {},
         attempts => $args->{reconnect_attemps} || 10,
-        time_between_attempts => $args->{time_between_attempts} || 5,
+        time_between_attempts => $args->{time_between_attempts} || 5, #seconds
 
     }, $class;
 } #new
@@ -166,7 +170,9 @@ sub stop {
     my $self = shift;
 
     _log( " with '$self->{listener_socket}' socket" );
-    $self->{listener_socket}->close if $self->{listener_socket};
+    if( $self->{listener_socket} ) {
+        $self->{listener_socket}->close;
+    }
 
     if( my $pid = $self->{server_pid} ) {
         $self->{error} = "Sending INT signal to lock server of pid '$pid'";
@@ -226,16 +232,18 @@ sub run {
 sub _create_listener_socket {
     my $self = shift;
 
-
     my( $listener_socket, $count );
 
-    my $st = time;
+    my $st = Time::HiRes::time;
     
     until( $listener_socket || $count++ > $self->{attempts} ) {
         $listener_socket = new IO::Socket::INET(
-            Listen    => 10,
-            Type      => SOCK_STREAM,
-            LocalAddr => "$self->{host}:$self->{port}",
+            Listen    => 1,
+            LocalPort => $self->{port},
+#            LocalAddr => "$self->{host}:$self->{port}",
+#            Proto     => 'tcp',
+#            ReuseAddr => 1,
+#            ReusePort => 1,
             );
         last if $listener_socket;
         print STDERR "Unable to open the lock server socket $@, $!. Retry $count of 10\n";
@@ -262,48 +270,55 @@ sub _create_listener_socket {
         _log( "lock server  : got INT signal. EXITING." );
         exit;
     };
-    setsockopt( $listener_socket, SOL_SOCKET, SO_KEEPALIVE, 1 );
     return $listener_socket;
 } #_create_listener_socket
 
 sub _run_loop {
     my( $self, $listener_socket ) = @_;
-    while( my $connection = $listener_socket->accept ) {
-        my $req = <$connection>; 
-        $req =~ s/\s+$//s;
-        _log( "lock server : incoming request : '$req'" );
-        # could have headers, but ignore those. Find \n\n
-        while( my $data = <$connection> ) {
-            chomp $data;
-            last unless $data =~ /\S/;
-        }
 
-        my( $cmd, $key, $locker_id ) = split( '/', substr( $req, 5 ) );
-        if( $cmd eq 'CHECK' ) {
-            $self->_check( $connection, $key );
-        } elsif( $cmd eq 'LOCK' ) {
-            $self->_lock( $connection, $key, $locker_id );
-        } elsif( $cmd eq 'UNLOCK' ) {
-            $self->_unlock( $connection, $key, $locker_id );
-        } elsif( $cmd eq 'VERIFY' ) {
-            $self->_verify( $connection, $key, $locker_id );
-        } elsif( $cmd eq 'PING' ) {
-            print $connection "1\n";
-            $connection->close;
-        } elsif( $cmd eq 'SHUTDOWN') {
-            if( $self->{allow_shutdown}) {
-                print $connection "1\n";
-                $connection->close;
-                $self->stop;
+    my $sel = IO::Select->new( $listener_socket );
+    my @ready;
+    while(@ready = $sel->can_read) {
+        for my $connection (@ready) {
+            if( $connection == $listener_socket ) {
+                $sel->add($listener_socket->accept );
             } else {
-                _log( "lock server : got shutdown request but not configured to allow it" );
+                my $req = <$connection>; 
+                $req =~ s/\s+$//s;
+                _log( "lock server : incoming request : '$req'" );
+                # could have headers, but ignore those. Find \n\n
+                while( my $data = <$connection> ) {
+                    chomp $data;
+                    last unless $data =~ /\S/;
+                }
+
+                my( $cmd, $key, $locker_id ) = split( '/', substr( $req, 5 ) );
+                if( $cmd eq 'CHECK' ) {
+                    $self->_check( $connection, $key );
+                } elsif( $cmd eq 'LOCK' ) {
+                    $self->_lock( $connection, $key, $locker_id );
+                } elsif( $cmd eq 'UNLOCK' ) {
+                    $self->_unlock( $connection, $key, $locker_id );
+                } elsif( $cmd eq 'VERIFY' ) {
+                    $self->_verify( $connection, $key, $locker_id );
+                } elsif( $cmd eq 'PING' ) {
+                    print $connection "1\n";
+                } elsif( $cmd eq 'SHUTDOWN') {
+                    if( $self->{allow_shutdown}) {
+                        print $connection "1\n";
+                        $connection->close;
+                        $self->stop;
+                    } else {
+                        _log( "lock server : got shutdown request but not configured to allow it" );
+                    }
+                } else {
+                    _log( "lock server : did not understand command '$cmd'" );
+                }
+                $sel->remove($connection);
                 $connection->close;
             }
-        } else {
-            _log( "lock server : did not understand command '$cmd'" );
-            $connection->close;
-        }
-    } #while still getting connections
+        } #ready loop
+    } #can_read loop
 } #_run_loop
 
 sub _check {
@@ -315,7 +330,7 @@ sub _check {
 
     
     #check for timed out lockers
-    my $t = time;
+    my $t = Time::HiRes::time;
     while( @$lockers && $t > $self->{_locker_counts}{$lockers->[0]}{$key_to_check} ) {
         _log( "lock server _check : '$key_to_check' timed out for locker '$lockers->[0]'" );
         if( 1 == keys %{ $self->{_locker_counts}{$lockers->[0]} } ) {
@@ -332,7 +347,6 @@ sub _check {
     } else {
         print $connection "0\n";
     }
-    $connection->close;
 }
 
 sub _log {
@@ -343,13 +357,12 @@ sub _log {
 
 sub _lock {
     my( $self, $connection, $key_to_lock, $locker_id ) = @_;
-
     _log( "lock request :  for '$locker_id' and key '$key_to_lock'" );
 
     $self->{_locks}{$key_to_lock} ||= [];
     my $lockers = $self->{_locks}{$key_to_lock};
     #check for timed out lockers
-    my $t = time;
+    my $t = Time::HiRes::time;
 
     while( @$lockers && $t > $self->{_locker_counts}{$lockers->[0]}{$key_to_lock} ) {
         _log( "lock '$key_to_lock' timed out for locker '$lockers->[0]'" );
@@ -364,12 +377,11 @@ sub _lock {
     if( 0 < (grep { $_ eq $locker_id } @$lockers) ) {
         _log( "lock request error. '$locker_id' already in the lock queue" );
         print $connection "0\n";
-        $connection->close;
         return;
     }
 
     # store when this times out 
-    my $timeout_time = time + $self->{lock_timeout};
+    my $timeout_time = Time::HiRes::time + $self->{lock_timeout};
     $self->{_locker_counts}{$locker_id}{$key_to_lock} = $timeout_time;
     push @$lockers, $locker_id;
 
@@ -389,13 +401,13 @@ sub _lock {
                 exit;
             };
             $SIG{HUP} = sub {
-                _log( "lock request : child got HUP, so is now locked. exiting" );
+                _log( "lock request : child got HUP, so is now locked." );
                 $connection->print( "$timeout_time\n" );
                 $connection->close;
                 exit;
             };
             _log( "lock request : child ready to wait" );
-            sleep $self->{lock_attempt_timeout};
+            usleep 1_000_000 * $self->{lock_attempt_timeout};
             _log( "lock request failed : child timed out" );
             print $connection "0\n";
             $connection->close;
@@ -404,7 +416,6 @@ sub _lock {
     } else {
         _log( "lock request : no need to invoke more processes. locking" );
         print $connection "$timeout_time\n";
-        $connection->close;
     }
 } #_lock
 
@@ -434,12 +445,10 @@ sub _unlock {
         }
         _log( "unlock : done, informing connection" );
         print $connection "1\n";
-        $connection->close;
     } else {
         _log( "unlock error : Wrong locker_id to unlock for unlock for locker '$locker_id' and key '$key_to_unlock'. The locker_id must be the one at the front of the queue" );
         # "Wrong locker_id to unlock. The locker_id must be the one at the front of the queue";
         print $connection "0\n";
-        $connection->close;
     }
 } #_unlock
 
@@ -452,7 +461,8 @@ sub _verify {
     my $lockers = $self->{_locks}{$key_to_check};
 
     #check for timed out lockers
-    my $t = time;
+    my $t = Time::HiRes::time;
+    _log( "verify:  compare '$t' > '$self->{_locker_counts}{$lockers->[0]}{$key_to_check}' if the first is greater, there is a time out" );
     while( @$lockers && $t > $self->{_locker_counts}{$lockers->[0]}{$key_to_check} ) {
         _log( "verify:  '$key_to_check' timed out for locker '$lockers->[0]'" );
         if( 1 == keys %{ $self->{_locker_counts}{$lockers->[0]} } ) {
@@ -468,7 +478,6 @@ sub _verify {
     } else {
         print $connection "0\n";
     }
-    $connection->close;
 }
 
 
@@ -650,6 +659,6 @@ __END__
 
 =head1 VERSION
 
-       Version 1.72  (June 1, 2016))
+       Version 1.73  (June 1, 2016))
 
 =cut
